#!/usr/bin/env python3
import pygor3 as p3
from optparse import OptionParser

def main():
    task = p3.IgorTask()
    parser = OptionParser(usage="usage: %prog [options] ")
    parser.add_option("-s", "--species", dest="species", help='Igor species')
    parser.add_option("-c", "--chain", dest="chain", help='Igor chain')
    parser.add_option("-b", "--batch", dest="batch", help='Batchname to identify run. If not set random name is generated')
    parser.add_option("-o", "--output", dest="output", help='filename of csv file to export data')
    parser.add_option("-i", "--input", dest="input", help='Input sequences filename to calculate pgen')

    (options, args) = parser.parse_args()

    if options.species == None:
        print("species is mandatory")
        exit()
    else:
        task.igor_species = str(options.species)

    if options.chain == None:
        print("species is mandatory")
        exit()
    else:
        task.igor_chain = str(options.chain)

    if options.input == None:
        print("input is mandatory")
        exit()
    else:
        input = options.input
        task.igor_read_seqs = input

    if options.output == None:
        print("output is mandatory")
        import os
        basename, extension = os.path.splitext(task.igor_read_seqs)
        output = basename+"_pgen.csv"
        # task.igor_read_seqs
        #exit()
    else:
        output = options.output

    if options.batch == None:
        print("Batchname not set. Random batchname will be assing.")

    if not len(args) == 0:
      print("Supply a correct filename")
      exit()

    task.update_batch_filenames()
    task.run_evaluate()

    # task = p3.IgorTask.load_from_batchname("testing")
    # task.igor_specie = "human"
    # task.igor_chain = "beta"
    # task.load_IgorModel()
    task.update_batch_filenames()
    task.load_IgorModel()
    df = task.get_pgen_pd()
    task.run_clean_batch()
    df.to_csv(output )

if __name__ == "__main__":
    main()


#!/usr/bin/env python3
import pygor3 as p3
import argparse

def get_CDR3_data(mdl, scen):
    df_V = mdl.genomic_dataframe_dict['V']
    mdl.parms.get_event_dict()
    print ( str(df_V[df_V['id'] == scen['v_choice']]['gene_name'].values) )

def main():
    parser = argparse.ArgumentParser()
    igor_models = parser.add_argument_group('IGoR default models')

    parser.add_argument("-D", "--database", dest="database", help="Igor database created with database script.",
                        default="Ajam.db")

    args = parser.parse_args()


    # Get the CDR3 from scenarios
    # 1. Load database and model within database
    db = p3.IgorSqliteDB.create_db(args.database)
    mdl = db.get_IgorModel()

    # 2. load alignment genomic dataframes inside mld instance
    mdl.set_genomic_dataframe_dict(db.get_IgorGenomicDataFrame_dict())
    gene_templates_dict = db.get_IgorGenomicDataFrame_dict()

    # Get the CDR3 length from a bs
    # For a particular seq_index
    seq_index = 5
    # 3. list all calculated scenarios
    scenarios_list = db.get_IgorBestScenarios_By_seq_index_IgorModel(seq_index, mdl)

    # 4. Get the alignments of the sequence
    v_aln_list = db.get_IgorAlignment_data_list_By_seq_index("V", seq_index)
    j_aln_list = db.get_IgorAlignment_data_list_By_seq_index("J", seq_index)
    # str_gene_name = "U66059|TRBV5-1*01|Homo sapiens|F|V-REGION|113806..114091|286 nt|1| | | | |286+0=286| | |"


    # 5. For each sequence get the alignments and calculate the anchor_in_read from the anchor_index and the alignment offset
    for aln in v_aln_list:
        # print(gene_templates_dict["V"])
        anchor_index = gene_templates_dict["V"].loc[aln.gene_id].anchor_index
        aln.anchor_in_read = anchor_index + aln.offset
        # print(" v: ", aln.gene_id, aln.score, aln.anchor_in_read, anchor_index, aln.offset, aln.strGene_name)

    for aln in j_aln_list:
        # print(gene_templates_dict["V"])
        #print(aln.gene_id,)
        anchor_index = gene_templates_dict["J"].loc[aln.gene_id].anchor_index
        aln.anchor_in_read = anchor_index + aln.offset
        # print(" j: ", aln.gene_id, aln.score, aln.anchor_in_read, anchor_index, aln.offset, aln.strGene_name)

    # print(v_aln_list[0].anchor_in_read)
    # print(j_aln_list[0].anchor_in_read)

    # 6. For each scenario get the CDR3
    # scenarios_list = db.get_IgorBestScenarios_By_seq_index_IgorModel(seq_index, mdl)
    for scen in scenarios_list:
        # scen = scenarios_list[0]
        print("---> Scenario", scen.scenario_rank)
        print('v_choice', scen['v_choice'])
        print('j_choice', scen['j_choice'])
        # FIXME : I'm assuming the same index for parms and gene templates
        v_anchor_index = gene_templates_dict["V"].loc[scen['v_choice']].anchor_index
        j_anchor_index = gene_templates_dict["J"].loc[scen['j_choice']].anchor_index
        # Now use the alignments to get the list of the alignments with 'v_choice' id a seq_index
        #print(gene_templates_dict["V"].loc[0].anchor_index)
        scen_v_alns = [v_aln.anchor_in_read for v_aln in v_aln_list if v_aln.gene_id == scen['v_choice']]
        scen_j_alns = [j_aln.anchor_in_read for j_aln in j_aln_list if j_aln.gene_id == scen['j_choice']]
        print(scen_v_alns)
        print(scen_j_alns)


    # print(gene_templates_dict["V"])



    # print(list( map(lambda x: (x.strGene_name, x.score, x.offset, x.anchor_in_read), lista)) )

    return 0

    # str_gene_name = "TRBD1*01"
    # print(lista.to_dict())
    lista02 = [aln for aln in lista if aln.strGene_name == str_gene_name]
    print(lista02[0].to_dict())
    # print(mdl.parms.Event_dict['v_choice'] ) #.iloc[0])

    # Make a correspondace table between model gene id and alignment gene id
    # IgorVGeneTemplate -> vgene_id
    # IgorER_v_choice -> id
    """
    SELECT t1.*, t2.* FROM IgorER_v_choice t1 INNER JOIN IgorVGeneTemplate t2 WHERE t1.name==t2.gene_name; 
    """

    # HOW TO GET THE ALIGNMENTS
    """
    SELECT t2.gene_name, t1.*, t2.sequence 
    FROM Igor{upper}Alignments t1  
    INNER JOIN Igor{upper}GeneTemplate t2 
    WHERE t1.seq_index==0 
    AND t1.{lower}gene_id == t2.{lower}gene_id 
    AND t2.gene_name=='TRBD1*01' 
    ORDER BY score DESC
    """

    """
    SELECT t2.gene_name, t1.*, t2.sequence 
    FROM IgorDAlignments t1  INNER JOIN IgorDGeneTemplate t2 
    WHERE t1.seq_index==0 AND t1.dgene_id == t2.dgene_id 
    ORDER BY score DESC
    """

    # TODO: IDEAL
    # scen['v_choice'].get_value(mdl)
    # scen['v_choice'].get_sequence(mdl)
    # scen.get_value('v_choice')
    # scen.get_name('v_choice')
    # scen.get_id('v_choice')


    # get_CDR3_data(mdl, scen)
    # scen = scenarios_list[0]
    # scen_dict = scen.realizations_ids_dict
    #
    # for event in mdl.parms.Event_list:
    #     if not (event.event_type == 'DinucMarkov'):
    #         scen_dict[event.nickname] = scen_dict.pop('id_' + event.nickname)


    # FIXME: DON'T KNOW WHY THE D GENES ARE
    #  A LITTLE BIT WEIRD.
    # strEvent = 'd_gene'
    # strEvent = 'd_3_del'
    # strEvent = 'd_5_del'
    # strEvent = 'vd_dinucl'
    # delta = mdl0.Pmarginal[strEvent] - mdl.Pmarginal[strEvent]
    # print("="*50)
    # print(mdl0.Pmarginal[strEvent])
    # print("-" * 50)
    # print(mdl.Pmarginal[strEvent])
    # print("-" * 50)
    # print(delta)

if __name__ == "__main__":
    main()



#!/usr/bin/env python3
import pygor3 as p3


# def get_V_anchor_index_in_seq_from_gene(align:p3.IgorAlignment_data):
#     ins_size = len(align.insertions)
#     dels_size = len(align.deletions)
#     dict_gene_anchor_index = dict()
#     # TODO: COUNT number of insertions before cdr3_v_gene_anch


#
# # TODO: THERE IS A WAY TO GET
# def generate_str_fasta(indexed_sequence, list_vdj_alignments:dict):
#     """ Given an Sequence index and the corresponding alignments vj/ vdj
#     return a string with considering only offset"""
#
#     indexed_sequence.sequence = indexed_sequence.sequence.lower()
#     # add mismatches in sequence.
#     s = list(indexed_sequence.sequence)
#     for key_align in list_vdj_alignments.keys():
#         for pos_mis in list_vdj_alignments[key_align].mismatches:
#             s[pos_mis] = s[pos_mis].upper()
#     indexed_sequence.sequence = "".join(s)
#
#     str_fasta = ""
#     min_offset_key = min(list_vdj_alignments.keys(), key=lambda x: list_vdj_alignments[x].offset)  # .offset
#     min_offset = list_vdj_alignments[min_offset_key].offset
#     min_offset = min(indexed_sequence.offset, min_offset)
#
#     delta_offset = indexed_sequence.offset - min_offset
#     str_prefix = '-' * (delta_offset)
#     str_fasta_sequence = str_prefix + indexed_sequence.sequence
#     print(str_fasta_sequence)
#     str_fasta = str_fasta + "> " + str(indexed_sequence.seq_index) + "\n"
#     str_fasta = str_fasta + str_fasta_sequence + "\n"
#     for key in list_vdj_alignments.keys():
#         list_vdj_alignments[key].strGene_seq = list_vdj_alignments[key].strGene_seq.lower()
#         delta_offset = list_vdj_alignments[key].offset - min_offset
#         str_prefix = '-' * (delta_offset)
#         str_fasta_sequence = str_prefix + list_vdj_alignments[key].strGene_seq
#         print(str_fasta_sequence)
#         str_fasta = str_fasta + "> " + key + ", " + list_vdj_alignments[key].strGene_name + "\n"
#         str_fasta = str_fasta + str_fasta_sequence + "\n"
#         offset_5_p = list_vdj_alignments[key].offset_5_p - min_offset
#         offset_3_p = list_vdj_alignments[key].offset_3_p - min_offset
#         print("delta_offset : ", delta_offset)
#         print("offset_5_p : ", list_vdj_alignments[key].offset_5_p, offset_5_p)
#         print("offset_3_p : ", list_vdj_alignments[key].offset_3_p, offset_3_p)
#         str_prefix_2 = '-' * (offset_5_p+1)
#         str_fasta_sequence2 = str_prefix_2 + str_fasta_sequence[offset_5_p+1:offset_3_p+1]
#
#         str_fasta = str_fasta + "> " + list_vdj_alignments[key].strGene_name + ", score : "+str(list_vdj_alignments[key].score) + "\n"
#         str_fasta = str_fasta + str_fasta_sequence2 + "\n"
#
#         # TODO ADD MISMATCHES
#         align = list_vdj_alignments[key]
#         # align mismatches are in indexed sequence reference I need to convert it to gene reference given the alignment
#         # given the align.offset
#         # pos_in_gene  = pos_in_seq - align.offset
#         # pos_in_gene = cdr3 - align.offset
#
#     return str_fasta

def generate_csv_line(indexed_sequence:p3.IgorIndexedSequence, indexed_cdr3_record:list, list_vdj_alignments:dict, sep=';',
                      header_list=['sequence_id', 'sequence', 'v_call', 'd_call', 'j_call', 'v_score', 'd_score', 'j_score']):
    csv_line = ""
    indexed_sequence.sequence = indexed_sequence.sequence.lower()
    fields_dict = dict()
    fields_dict['sequence_id'] = str(indexed_sequence.seq_index)
    fields_dict['sequence'] = indexed_sequence.sequence
    fields_dict['v_call'] =  list_vdj_alignments['V'].strGene_name
    fields_dict['d_call'] = list_vdj_alignments['D'].strGene_name
    fields_dict['j_call'] = list_vdj_alignments['J'].strGene_name
    fields_dict['v_score'] = str(list_vdj_alignments['V'].score)
    fields_dict['d_score'] = str(list_vdj_alignments['D'].score)
    fields_dict['j_score'] = str(list_vdj_alignments['J'].score)
    # FIXME: CREATE A BETTER WAY TO DO THIS
    fields_dict['v_anchor'] = ""
    fields_dict['j_anchor'] = ""
    fields_dict['junction'] = ""
    fields_dict['junction_aa'] = ""
    try:
        fields_dict['v_anchor'] = str(indexed_cdr3_record[1]) #""
        fields_dict['j_anchor'] = str(indexed_cdr3_record[2]) #""
        fields_dict['junction'] = str(indexed_cdr3_record[3]) #""
        fields_dict['junction_aa'] = str(indexed_cdr3_record[4]) #""
    except Exception as e:
        print("No junction for sequence : "+fields_dict['sequence_id'])
        print(e)
        pass

    # align = p3.IgorAlignment_data()

    for field in header_list:
        csv_line = csv_line + fields_dict[field]+sep
    return csv_line

    # mixcr :
    # targetSequences	targetQualities	allVHitsWithScore
    # allDHitsWithScore	allJHitsWithScore	allCHitsWithScore	allVAlignments	allDAlignments
    # allJAlignments	allCAlignments	nSeqFR1	minQualFR1	nSeqCDR1	minQualCDR1	nSeqFR2	minQualFR2	nSeqCDR2	minQualCDR2	nSeqFR3	minQualFR3	nSeqCDR3	minQualCDR3	nSeqFR4	minQualFR4	aaSeqFR1	aaSeqCDR1	aaSeqFR2	aaSeqCDR2	aaSeqFR3	aaSeqCDR3	aaSeqFR4	refPoints

    # AIRR :
    # sequence_id (req)
    # sequence (req)
    # rev_comp (req) //false by default # FIXME: for now
    # productive (req) // false by default # FIXME: a naive alignmnet of productive is possible.
    # locus (chain type) // eg. TCR IG
    # v_call
    # d_call
    # d2_call // FIXME: not gonna use this by the moment.
    # j_call
    # sequence alignment // FIXME: Aligned portion of query sequence. IMGT-gaps check the best way to do this
    # germline_alignment // FIXME: Assembled, aligned, full-length inferred germline sequence spanning the same region as the sequence_alignment field (typically the V(D)J region) and including the same set of corrections and spacers (if any)
    # junction
    # junction_aa // Our CDR3
    # np1 (nts btwn V and D or btwn V and J  // vd_ins  or vj_ins
    # np2 (nts btwn first D and J gene or btwn first D and second D // dj_ins or d1d2_ins
    # np3 (nts btwn second D and J gene
    # v_score
    # v_cigar
    # d_score
    # d_cigar
    # j_score
    # j_cigar
    # junction_length




def generate_str_fasta_simple(indexed_sequence, list_vdj_alignments:dict):
    """ Given an Sequence index and the corresponding alignments vj/ vdj
    return a string with considering only offset"""

    str_fasta = ""
    str_fasta = str_fasta+"> "+ str(indexed_sequence.seq_index) + "\n"
    str_fasta = str_fasta+indexed_sequence.sequence+"\n"
    for key in list_vdj_alignments.keys():
        str_fasta = str_fasta + "> " + key + ", " + list_vdj_alignments[key].strGene_name + "\n"
        str_fasta = str_fasta + list_vdj_alignments[key].strGene_seq + "\n"

    # print(list_vdj_alignments['V'])
    # print(list_vdj_alignments['D'])
    return str_fasta



import argparse

def main():

    parser = argparse.ArgumentParser()
    # parser.add_argument("-b", "--batch", dest="batch", help='Batchname to identify run. If not set random name is generated')
    parser.add_argument("-o", "--output", dest="output", help='filename of csv file to export data')
    # parser.add_argument("-g", "--path_ref_genome", dest="path_ref_genome", help='Directory where genome references are stored: genomicDs.fasta,  genomicJs.fasta,  genomicVs.fasta,  J_gene_CDR3_anchors.csv,  V_gene_CDR3_anchors.csv', default='./ref_genome')
    # parser.add_argument("-w", "--WD", dest="working_directory", help="Path where files gonna be created.", default='./')
    parser.add_argument("-D", "--database", dest="database", help="Igor database created with database script.")
    # parser.add_argument("")

    args = parser.parse_args()


    # # load gene templates
    # genomes = p3.IgorRefGenome()
    # genomes.path_ref_genome = args.path_ref_genome # "/home/alfaceor/Dropbox/PosDoc/IGoR/dev/MyGithub/pygor3/demo/thi/genomics_repseqio_F"
    # genomes.update_fln_names()
    # genomes.load_dataframes()


    # task = p3.IgorTask()
    # task.igor_wd = args.working_directory #"/home/alfaceor/Dropbox/PosDoc/IGoR/dev/MyGithub/pygor3/demo/thi/test"
    # task.igor_batchname = args.batch #"sample"
    #
    # task.update_batch_filenames()

    db = p3.IgorSqliteDB()
    #db.flnIgorDB = task.igor_wd+"/"+task.igor_batchname+".db"
    db.flnIgorDB = args.database
    db.connect_db()

    # Make a loop over all sequences
    if args.output is None:
        fln_output = args.database.split(".db")[0]+"_na.csv"
    else:
        fln_output = args.output

    ofile = open(fln_output, 'w')
    seq_index_list = db.execute_select_query("SELECT seq_index FROM IgorIndexedSeq;")
    seq_index_list = map(lambda x:x[0], seq_index_list)
    header_list = ['sequence_id', 'sequence', 'v_call', 'd_call', 'j_call', 'v_score', 'd_score', 'j_score', 'v_anchor', 'j_anchor', 'junction', 'junction_aa']
    sep = ";"
    str_header = sep.join(header_list)
    ofile.write(str_header+'\n')
    for seq_index in seq_index_list:
        try:
            # seq_index = args.seq_index
            indexed_sequence = db.get_IgorIndexedSeq_By_seq_index(seq_index)
            indexed_sequence.offset = 0

            indexed_cdr3_record = db.fetch_IgorIndexedCDR3_By_seq_index(seq_index)
            print(indexed_cdr3_record)

            best_v_align_data = db.get_best_IgorAlignment_data_By_seq_index('V', indexed_sequence.seq_index)
            best_d_align_data = db.get_best_IgorAlignment_data_By_seq_index('D', indexed_sequence.seq_index)
            best_j_align_data = db.get_best_IgorAlignment_data_By_seq_index('J', indexed_sequence.seq_index)

            vdj_naive_alignment = {'V': best_v_align_data,
                               'D': best_d_align_data,
                               'J': best_j_align_data}

            v_align_data_list = db.get_IgorAlignment_data_list_By_seq_index('V', indexed_sequence.seq_index)
            # print('V', len(v_align_data_list), [ ii.score for ii in v_align_data_list])
            try:
                d_align_data_list = db.get_IgorAlignment_data_list_By_seq_index('D', indexed_sequence.seq_index)
            except Exception as e:
                print(e)
                print("No D sequences alignments found!")
                pass
            # print('D', len(d_align_data_list), [ ii.score for ii in d_align_data_list])
            j_align_data_list = db.get_IgorAlignment_data_list_By_seq_index('J', indexed_sequence.seq_index)
            # print('J', len(j_align_data_list), [ ii.score for ii in j_align_data_list])

            # 1. Choose the highest score then check if this one is the desire range.
            # if there is an overlap
            # calculate score without overlap. If overlap
            # if hightest score
            for i, d_align_data in enumerate(d_align_data_list):
                # Check if D is btwn V and J position
                if (best_v_align_data.offset_3_p <= d_align_data.offset_5_p) and (d_align_data.offset_3_p <= best_j_align_data.offset_5_p) :
                    #vdj_naive_alignment['D'+str(i)] = d_align_data
                    vdj_naive_alignment['D'] = d_align_data
                    break

            # ofile = open(task.igor_batchname+'__'+str(indexed_sequence.seq_index)+'_na.fasta', 'w')
            # str_fasta = generate_str_fasta(indexed_sequence, vdj_naive_alignment)
            # ofile.write(str_fasta)

            str_csv_line = generate_csv_line(indexed_sequence, indexed_cdr3_record, vdj_naive_alignment, sep=sep, header_list=header_list)
            ofile.write(str_csv_line+"\n")
        except Exception as e:
            print("balsa : "+str(seq_index))
            print(e)
            pass
    ofile.close()

    # given a IgorGenomic template and using pygor database for IGoR get anchors from alignment and then

    # algn_fln = 'sample__2_na.fasta'
    # seq_lens = [len(record.seq) for record in SeqIO.parse(algn_fln, 'fasta')]
    # max_seq_lens = max(seq_lens)
    # new_alignments = list()
    # for record in SeqIO.parse(algn_fln, 'fasta'):
    #     str_extra = max_seq_lens - len(record.seq)
    #     record.seq = record.seq + "-" * str_extra
    #     new_alignments.append(record)
    #     # print(len(record.seq), max_seq_lens)
    #
    # tmp_fln = 'example.fasta'
    # SeqIO.write(new_alignments, tmp_fln, "fasta")

    # da_heavy_pen_nuc44 = p3.utils.da_heavy_pen_nuc44_vect
    # print(da_heavy_pen_nuc44)
    # import pandas as pd
    # df = pd.DataFrame(data=da_heavy_pen_nuc44.values, index=da_heavy_pen_nuc44['lbl__' + 'x'].values,
    #                   columns=da_heavy_pen_nuc44['lbl__' + 'y'].values)
    #
    # print(df)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
import pygor3 as p3
import argparse

def generate_event_delta_Kronecker(event_nickname, event_id):
    def tmp_funct(bs:p3.IgorScenario):
        if bs[event_nickname] == event_id:
            return 1
        else:
            return 0
    return tmp_funct

def calc_average(db:p3.IgorSqliteDB, observable_function):
    for sigma in db.fetch_IgorIndexedSeq_indexes():
        print(sigma, len(db.fetch_IgorIndexedSeq_indexes()))


# from  optparse import OptionParser
def main():
    #parser = OptionParser()
    #parser.add_option("-s", "--species", dest="species", help='Igor species')
    #parser.add_option("-c", "--chain", dest="chain", help='Igor chain')
    #parser.add_option("-b", "--batch", dest="batch", help='Batchname to identify run. If not set random name is generated')
    #parser.add_option("-o", "--output", dest="output", help='filename of csv file to export data')

    parser = argparse.ArgumentParser()
    igor_models = parser.add_argument_group('IGoR default models')

    # Use IGoR default model
    igor_models.add_argument("-s", "--species", dest="species", help='Igor species')
    igor_models.add_argument("-c", "--chain", dest="chain", help='Igor chain')  # , type=str, choices=['TRB', 'TRA'])

    parser.add_argument("-D", "--database", dest="database", help="Igor database created with database script.")

    igor_models = parser.add_argument_group('IGoR default models')

    #(options, args) = parser.parse_args()
    # Create an IgorModel
    mdl = p3.IgorModel.load_default("human", "tcr_beta")
    da = mdl.xdata['v_choice']
    import numpy as np

    db = p3.IgorSqliteDB.create_db("Ajam.db")
    db.gen_IgorBestScenarios_cols_list()
    print(db.sql_IgorBestScenarios_cols_name_type_list)
    record = db.fetch_IgorBestScenarios_By_seq_index(0)[0]
    bs = p3.IgorScenario.load_FromSQLRecord(record, db.sql_IgorBestScenarios_cols_name_type_list)
    print(record)
    print(bs)
    print(bs.to_dict())
    func_V_delta = generate_event_delta_Kronecker('id_v_choice', 49)
    func_J_delta = generate_event_delta_Kronecker('id_j_choice', 0)
    # Now I have a kronecker delta function
    f = func_V_delta(bs)*func_J_delta(bs)
    print(f)

    func_V_delta = generate_event_delta_Kronecker('id_v_choice', 49)
    func_J_delta = generate_event_delta_Kronecker('id_j_choice', 0)

    db = p3.IgorSqliteDB.create_db("Ajam.db")
    db.gen_IgorBestScenarios_cols_list()
    indexes_list = db.fetch_IgorIndexedSeq_indexes()
    print("len: ", len(indexes_list))
    function_average = 0
    for indx in indexes_list:
        # aln_data = db.get_IgorAlignment_data_list_By_seq_index('V', indx)
        bs_data_list = db.fetch_IgorBestScenarios_By_seq_index(indx)
        tmp_average = 0
        scenario_norm_factor = 0
        for bs_data in bs_data_list:
            bs = p3.IgorScenario.load_FromSQLRecord(bs_data, db.sql_IgorBestScenarios_cols_name_type_list)
            tmp_average = tmp_average + bs.scenario_proba_cond_seq*func_V_delta(bs)*func_J_delta(bs)
            scenario_norm_factor = scenario_norm_factor + bs.scenario_proba_cond_seq
        tmp_average = tmp_average/scenario_norm_factor
        function_average = function_average + tmp_average

    print( function_average/len(indexes_list) )
    # Insert normalized probability in new table
    return 0




    # Make a query with the variables used for the specification
    # TODO: CORRECT FUNCTION OF CUMULATE PROBABILITY
    observable_vars_value = {
        'd_gene': " TRBD2*01", # id is 1
        'd_5_del': 4 # id is 8
    }
    # FIRST TRY TO WITH NAME IF NAME IS EMPTY THEN change it to value
    # TODO: return dict with the corresponding ids.
    # observable_ids = mdl.get_ids_dict_from_values(observable_vars_value)
    observable_ids = {
        'id_d_gene': [1],  # id is 1
        'id_d_5_del': [4]  # id is 8
    }

    records = db.fetch_IgorBestScenarios_By_events_dict(observable_ids)
    print( db.sql_cols_list )
    # For each record use a mdl to
    for record in records:
        print(record)

    print("*"*50)
    print(db.sql_cols_list)
    "SELECT name, type FROM pragma_table_info('IgorBestScenarios');"
    # ppp = db.execute_select_query("PRAGMA table_info(IgorBestScenarios);")
    ppp = db.execute_select_query("SELECT name, type FROM pragma_table_info('IgorBestScenarios');")
    db.sql_cols_list = db.execute_select_query("SELECT name, type FROM pragma_table_info('IgorBestScenarios');")

    # FIXME: but this is with database ids I want it with mdl ids,
    #  they are the same but is not the same 'v_choice' or 'id_v_choice'
    db.gen_IgorBestScenarios_cols_list()
    print(db.sql_IgorBestScenarios_cols_name_type_list)
    sce = p3.IgorScenario.load_FromSQLRecord(record, db.sql_cols_list)

    for ii, (col_name, tipo) in enumerate(db.sql_cols_list):
        if col_name == 'seq_index' :
            sce.seq_index = int(record[ii])
        elif col_name == 'scenario_rank' :
            sce.scenario_rank = int(record[ii])
        elif col_name == 'scenario_proba_cond_seq' :
            sce.scenario_proba_cond_seq = float(record[ii])
        else :
            if tipo == 'integer':
                sce.realizations_ids_dict[col_name] = int(record[ii])
            else:
                sce.realizations_ids_dict[col_name] = eval(record[ii])
        print(ii, col_name, tipo, record[ii])

if __name__ == "__main__":
    main()



#!/usr/bin/env python3

# get data from IMGT and generate a model
# options.type   = "VDJ"
# options.gene   = "TRB"
# options.species = "Mus+musculus"
import pygor3 as p3

def main():
    from optparse import OptionParser
    parser = OptionParser(usage="usage: %prog [options] ")
    parser.add_option("-t", "--type",   dest="type",   help="VJ or VDJ")
    parser.add_option("-c", "--chain",   dest="chain",   help="Type of chain like TRB")
    parser.add_option("-s", "--species", dest="species", help="Type of species")

    (options, args) = parser.parse_args()

    species_list = p3.imgt.get_species_list()
    species_list.remove('')

    if options.species is None:
        print("species is a mandatory option. Please select one species from list:")
        print(species_list)
    else:
        if not (options.species in species_list):
            print("Species not recognized. Please select one species from list:")
            print(species_list)
            exit()

    if options.type == "VDJ":

        flnVGenome = p3.imgt.download_gene_template(options.species, options.chain + 'V')
        flnDGenome = p3.imgt.download_gene_template(options.species, options.chain + 'D')
        flnJGenome = p3.imgt.download_gene_template(options.species, options.chain + 'J')

        # FIXME: ONCE THE GENE TEMPLATES ARE DOWNLOADED CHANGE THE NAME TO
        print(flnVGenome)
        # write anchors
        p3.imgt.download_genes_anchors(options.species, options.chain, flnVGenome, flnJGenome)

        # TODO: filter sequences for OLGA compatibility
        strGene = options.chain +"V"
        urlV_2CYS = p3.imgt.get_genedb_query81_imgtlabel(options.species, strGene, imgtlabel="2nd-CYS")
        dict2CYS = p3.imgt.genAnchDict(urlV_2CYS)
        list2CYS = list(dict2CYS.keys())

        v_genomes_list = p3.imgt.load_records_from_fasta(flnVGenome)
        v_genomes_trim_list = list()
        for v_genome in v_genomes_list:
            if v_genome in list2CYS:
                v_genomes_trim_list.append(v_genome)
            else:
                print(v_genome)
        p3.imgt.save_records2fasta(v_genomes_trim_list, flnVGenome + "_trim")

        # J-PHE
        urlJ_PHE = p3.imgt.get_genedb_query81_imgtlabel(options.species, options.species + "J", imgtlabel="J-PHE")
        dictJ_PHE = p3.imgt.genAnchDict(urlJ_PHE)
        listJ_PHE = list(dictJ_PHE.keys())

        # J-TRP
        urlJ_TRP = p3.imgt.get_genedb_query81_imgtlabel(options.species, options.species + "J", imgtlabel="J-TRP")
        dictJ_TRP = p3.imgt.genAnchDict(urlJ_TRP)
        listJ_TRP = list(dictJ_TRP.keys())

        j_genomes_list = p3.imgt.load_records_from_fasta(flnJGenome)
        j_genomes_trim_list = list()
        for j_genome in j_genomes_list:
            if j_genome in listJ_PHE:
                j_genomes_trim_list.append(j_genome)
            elif j_genome in listJ_TRP:
                j_genomes_trim_list.append(j_genome)
            else:
                print(j_genome)

        p3.imgt.save_records2fasta(j_genomes_trim_list, flnJGenome+"_trim")

    elif options.type == "VJ":
        flnVGenome = p3.imgt.download_gene_template(options.species, options.chain + 'V')
        flnJGenome = p3.imgt.download_gene_template(options.species, options.chain + 'J')
        # write anchors
        p3.imgt.download_genes_anchors(options.species, options.chain, flnVGenome, flnJGenome)
        # Now construct the models from a dictionary.

        # TODO EDIT LONG NAMES
        urlV_2CYS = p3.imgt.get_genedb_query81_imgtlabel(options.species, options.chain + "V", imgtlabel="2nd-CYS")
        dict2CYS = p3.imgt.genAnchDict(urlV_2CYS)
        list2CYS = list(dict2CYS.keys())

        v_genomes_list = p3.imgt.load_records_from_fasta(flnVGenome)
        print('*'*50)
        print("v_genomes_list : ", len(v_genomes_list))
        v_genomes_trim_list = list()
        for v_genome in v_genomes_list:
            if p3.imgt.genKey(v_genome.description) in list2CYS:
                v_genomes_trim_list.append(v_genome)
            else:
                print(v_genome.description)
        print("v_genomes_trim_list : ", len(v_genomes_trim_list))
        p3.imgt.save_records2fasta(v_genomes_trim_list, flnVGenome + "_trimmed")

        # J-PHE
        urlJ_PHE = p3.imgt.get_genedb_query81_imgtlabel(options.species, options.chain + "J", imgtlabel="J-PHE")
        dictJ_PHE = p3.imgt.genAnchDict(urlJ_PHE)
        listJ_PHE = list(dictJ_PHE.keys())

        # J-TRP
        urlJ_TRP = p3.imgt.get_genedb_query81_imgtlabel(options.species, options.chain + "J", imgtlabel="J-TRP")
        dictJ_TRP = p3.imgt.genAnchDict(urlJ_TRP)
        listJ_TRP = list(dictJ_TRP.keys())

        j_genomes_list = p3.imgt.load_records_from_fasta(flnJGenome)
        print('*' * 50)
        print("j_genomes_list : ", len(j_genomes_list))
        j_genomes_trim_list = list()
        for j_genome in j_genomes_list:
            if p3.imgt.genKey(j_genome.description) in listJ_PHE:
                j_genomes_trim_list.append(j_genome)
            elif p3.imgt.genKey(j_genome.description) in listJ_TRP:
                j_genomes_trim_list.append(j_genome)
            else:
                print(j_genome.description)
        print("j_genomes_trim_list : ", len(j_genomes_trim_list))

        p3.imgt.save_records2fasta(j_genomes_trim_list, flnJGenome + "_trimmed")
    else:
        print("type not recognized. Please choose VDJ or VJ.")




    # igor_species="human"
    # igor_chain="tcr_beta"
    # mdl = p3.IgorModel.load_default(igor_species, igor_chain)
    #
    # mdl.parms.plot_Graph()
    # mdl.get_events_nicknames_list()
    # mdl.plot_Event_Marginal('v_3_del')
    # mdl.plot_Event_Marginal('v_choice')
    #
    # mdl.xdata['v_choice']


if __name__ == "__main__":
    main()

#mdl.plot_Event_Marginal('v_choice')
# From the loaded model I want a list of event_types


"""
mdl.xdata['d_gene']
mdl.xdata['j_choice']

mdl.xdata['d_gene'].dot( mdl.xdata['j_choice'] )

mdl.xdata['d_3_del'].shape
mdl.xdata['d_3_del'].coords
mdl.xdata['d_3_del'].dims
mdl.xdata['d_3_del']*mdl.xdata['d_5_del']
(mdl.xdata['d_3_del'].dot(mdl.xdata['d_5_del'])).dims
mdl.parms.G['d_3_del']

elist = mdl.parms.Event_list
print(str(elist[0]))
nelist = sorted(elist, key=lambda x:x.priority, reverse=False)

mdl.parms.dictNicknameName.keys()
#0 Define the templates and define model.
# p3.imgt.download_gene(specie, gene)


#0.1 Get templates from IMGT from a list of available templates.
#1 Choose an specie and chain, this means define a model.

iseq = p3.IgorIndexedSequence(10,'ATCTGAGCT')
print(iseq)


mdl = p3.IgorModel.load_default("mouse", "tcr_beta")


#2 Input a sequence
#3 Align the sequence given the model, show the alignment
#3 Give me the pgen of the sequence probability of this sequence.


bs = p3.IgorBestScenariosVDJ()
print(bs)

bs.
"""
